// This file is part of BUDEES, a software that allows fast simulation of bubble dynamics in electrolyzers and their interactions with electrical parameters. 
// 
// Copyright (C) 2024 <CNRS-ENS Rennes-Université de Rennes-CY Cergy Paris Université-Université Paris Saclay-CNAM Paris-ENS Paris-Saclay>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with this program. If not, see https://www.gnu.org/licenses/

struct JNICaller {};
#include "Simulateur/Simulateur.h"


int main() {
	// -------- Parameters -------- //
	Configuration configuration;
	// System dimensions [m]
	configuration.L[0] = 2.64e-3;
	configuration.L[1] = 4.2e-3;
	// Current density [A/m2]
	configuration.J = 500;
	// Simulation duration [s]
	configuration.tf = 4.0;//24.25;
	// --------- Topology --------- //
	Topology topology;
	// Polygon count
	topology.polygonCount[0] = 1;
	topology.polygonCount[1] = 1;
	// Polygon indexs
	topology.polygonIndex[0] = new int[] {0, 6};
	topology.polygonIndex[1] = new int[] {6, 12};
	// Polygon defs
	Point points[] = {
			{0.0, 0.0}, {0.27, 0.0}, {0.44, 0.63}, {0.43, 0.8}, {0.27, 1.0}, {0.0, 1.0},
			{1.0, 0.0}, {1.0, 1.0}, {0.60, 1.0}, {0.72, 0.8}, {0.71, 0.46}, {0.60, 0.0},
	}; topology.polygonDefinitions = points;

	// Simulation
	if (!fs::exists(outputFolder)) {fs::create_directory(outputFolder);} outputDelete(outputFolder.c_str());
	Performances performances = simulate(configuration, topology, nullptr);
	cout << endl << "Simulation complete (" << (performances.collected[H2]+performances.lost[H2]+performances.remaining[H2]) << "mol of H2 produced) (n=" << (performances.collected[H2] / (performances.collected[H2]+performances.lost[H2])) << "%, R=" << performances.resistor << "ohm.m2, p=" << (performances.collected[H2] / (performances.collected[H2]+performances.contamination[H2])) << ") !" << endl;

	// Cleanup memory
	delete[] topology.polygonIndex[0];
	delete[] topology.polygonIndex[1];

	// Exit
	return 0;
}
