// This file is part of BUDEES, a software that allows fast simulation of bubble dynamics in electrolyzers and their interactions with electrical parameters. 
// 
// Copyright (C) 2024 <CNRS-ENS Rennes-Université de Rennes-CY Cergy Paris Université-Université Paris Saclay-CNAM Paris-ENS Paris-Saclay>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with this program. If not, see https://www.gnu.org/licenses/


/* --------- Velocity correction ---------- */
/* ---------------------------------------- */
void bubbleVelocityCorrection(double Ly, Bubble* it, bool* stopFlag);
bool bubbleVelocitySub(double Ly, Bubble* it, bool mode, WallIndicator** indicatorDone, Bubble** bubbleDone, bool* stopFlag) {
	// avec les murs
	for (int kk = 0; kk < wallMaxmimumIndicators; kk++) {
		WallIndicator *indicator = &it->interactor.walls[kk];
		if (!mode) if (indicator == *indicatorDone) continue;
		if (indicator->distance < (it->r*interactionSafetyHalfRange)) {
			double proj0 = it->ux*indicator->n.x + it->uy*indicator->n.y;
			if (proj0 < 0) { // la bulle se déplace vers le mur ?
				if (mode) {
					it->ux = it->ux - proj0*indicator->n.x;
					it->uy = it->uy - proj0*indicator->n.y;
					*indicatorDone = indicator;
					return true;
				} else {
					// blocage définitif ! -> arrêt
					it->ux = 0; it->uy = 0;
				}
			}
		}
	}
	// avec les autres spheres
	for (int kk = 0; kk < it->interactor.count; kk++) {
		if (it->interactor.bubbles[kk]->r != 0)
		if (it->interactor.distancesSqNorm[kk] < interactionSafetyRangeSq) {
			Bubble* kt = it->interactor.bubbles[kk];
			int kMode = it->interactor.imagedModes[kk];
			if (!mode) if (kt == *bubbleDone) continue;
			if ((kt->statCollision == 0) && (it->interactor.interactionIndexs[kk] != -1)) {
				// si c'est une bulle détachée non traitée, il faut lui fixer sa vitesse avant
				bubbleVelocityCorrection(Ly, kt, stopFlag);
			}
			double P21x = (it->x - kt->x), P21y = (it->y - yRepeat(kt->y, Ly, kMode));
			double proj1 = it->ux*P21x + it->uy*P21y;
			if (proj1 < 0) { // la bulle considérée se déplace vers une autre ?
				double proj2 = 0;
				if (kt->statCollision == 2) proj2 = kt->ux*P21x + kt->uy*P21y;
				if (proj1 < proj2) { // le mouvement relatif est-il bien un rapprochement ?
					if (mode) {
						double proj = (proj1-proj2)/(P21x*P21x + P21y*P21y);
						it->ux = it->ux - proj*P21x;
						it->uy = it->uy - proj*P21y;
						*bubbleDone = kt;
						return true;
					} else {
						double k = MAX(0, proj2 / proj1);
						it->ux = k*it->ux;
						it->uy = k*it->uy;
					}
				}
			}
		}
	}
	return false;
}
void bubbleVelocityCorrection(double Ly, Bubble* it, bool* stopFlag) {
	if (it->statCollision != 0) return;
	it->statCollision = 1;

	WallIndicator* indicatorDone = nullptr; Bubble* bubbleDone = nullptr;
	if (bubbleVelocitySub(Ly, it, true, &indicatorDone, &bubbleDone, stopFlag)) {
		bubbleVelocitySub(Ly, it, false, &indicatorDone, &bubbleDone, stopFlag);
	}

	it->statCollision = 2;
}
/* ---------------------------------------- */
